/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.clm;

import java.util.*;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.event.CropProviderEvent;
import nl.cloudfarming.client.crop.model.event.CropProviderEventKey;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.openide.util.*;

/**
 * Scheduler class to schedule updates from the web services
 * Will schedule a job every UPDATE_INTERVAL
 * 
 * @author Merijn Zengers
 */
class ClmUpdater implements LookupListener {
    //The delay between jobs 24 hours = 24 * 60 * 60 * 1000 = 86400000 (12 hours = 12 * 60 * 60 * 1000 = 43200000)
    protected static final int UPDATE_INTERVAL_MS = 86400000;
    private Timer timer = new Timer();
    protected static final String LAST_UPDATED_PROPERTY_NAME = "clmLastUpdated"; 
    private static final long DEFAULT_LAST_UPDATED_VALUE = 0;
    
    private ClmCropProvider clmCropProvider;
            
    ClmUpdater(ClmCropProvider clmCropProvider){
        this.clmCropProvider = clmCropProvider;
    }
    /**
     * Initializes the scheduler.
     * Checks if the last update is done within the last 24 hours
     * will schedule an update with a delay of 24 hours minus time since last update
     * and a interval of 24 hours
     */
    void init(){
        Lookup.getDefault().lookupResult(ClmCropService.class).addLookupListener(this);
        long lastUpdated = getStoredLastUpdated();
        long now = new Date().getTime();
        long timeSinceLastUpdate = now - lastUpdated;
        if(timeSinceLastUpdate > UPDATE_INTERVAL_MS){
            schedule(0);
        } else {
            schedule(UPDATE_INTERVAL_MS - timeSinceLastUpdate);
        }
    }
    
    /**
     * Returns when the web services where last updated
     * @return long the time since last update in milliseconds from epoch
     */
    long getStoredLastUpdated(){
        return NbPreferences.forModule(this.getClass()).getLong(LAST_UPDATED_PROPERTY_NAME, DEFAULT_LAST_UPDATED_VALUE);
    }
    
    /**
     * Schedules a new execution.
     * Will set the lastUpdated property to now
     * @param the delay for first execution 
     */
    void schedule(long delay){
        long time = new Date().getTime();
        NbPreferences.forModule(this.getClass()).putLong(LAST_UPDATED_PROPERTY_NAME, time);
        timer.schedule(new ClmUpdaterTask(), delay, UPDATE_INTERVAL_MS);
    }

    @Override
    public void resultChanged(LookupEvent ev) {
        timer.schedule(new ClmUpdaterTask(), new Date());
    }
    
    /**
    * Worker class to update the crop lists or create new ones from
    * data acquired from the web services
    *
    * @author Merijn Zengers
    */
    @NbBundle.Messages({
        "clm_updater.crop.source_id=CLM"
    })
    private class ClmUpdaterTask extends TimerTask {

       /**
        * Will get all crops from the CLM webservice through the ClmCropService and set the list of crops in the ClmCropProvider
        */
        @Override
        public void run() {
            List<Crop> crops = new ArrayList<>();

            // Find all ClmCropService classes
            Collection<? extends ClmCropService> cropServices = Lookup.getDefault().lookupAll(ClmCropService.class);
            for (ClmCropService clmService : cropServices) {
                for (String cropName : clmService.getCropNames()) {
                    // TODO: This will create crops in one locale, but will of course not translate existing crop created in another locale...
                    Crop crop = new Crop(NbBundle.getMessage(ClmUpdaterTask.class, "clm_updater.crop.source_id"));
                    crop.setName(cropName);
                    crops.add(crop);
                }
            }
            
            // Insert crops into Provider
            clmCropProvider.setCrops(crops);
            
            // Trigger 'Updated'-event
            new CropProviderEventProducer().triggerEvent(new CropProviderEvent(clmCropProvider.getId(), CropProviderEventKey.UPDATED)); 
        }
    }
    
    private class CropProviderEventProducer extends BaseGuiEventProducer {

        public CropProviderEventProducer() {
            super(CropProviderEventKey.class);
        }
    }
}

